PrimeFacesExt = {
    /**
     * Gets the extension of the current request URL.
     *
     * @return {string} The URL extensions.
     */
	getRequestUrlExtension : function() {
		return PrimeFacesExt.getUrlExtension(location.href);
	},	
		
    /**
     * Gets the extension of the given URL.
     *
     * @return {string} The URL extensions.
     */
	getUrlExtension : function(url) {
		return (url = url.substr(1 + url.lastIndexOf("/")).split('?')[0]).substr(url.lastIndexOf("."));
	},

    /**
     * Builds a resource URL for given parameters.
     * 
     * @param {string} name The name of the resource. For example: /core/core.js
     * @param {string} library The library of the resource. For example: primefaces-extensions
     * @param {string} version The version of the library. For example: 0.2.0-SNAPSHOT
     * 
     * @return {string} The resource URL.
     */
	getFacesResource : function(name, library, version) {
		var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
        
		scriptURI = scriptURI.replace('/primefaces-extensions.js', name);
		
		if (PrimeFacesExt.useUncompressedResources) {
			scriptURI = scriptURI.replace('ln=' + PrimeFacesExt.RESOURCE_LIBRARY_UNCOMPRESSED, 'ln=' + library);
		} else {
			scriptURI = scriptURI.replace('ln=' + PrimeFacesExt.RESOURCE_LIBRARY, 'ln=' + library);
		}

		var extractedVersion = RegExp('[?&]v=([^&]*)').exec(scriptURI)[1];
		if (version) {
			scriptURI = scriptURI.replace('v=' + extractedVersion, 'v=' + version);
		} else {
			scriptURI = scriptURI.replace('v=' + extractedVersion, '');
		}

        return location.protocol + '//' + location.host + scriptURI;
	},

    /**
     * Gets the version of the current PrimeFaces Extensions library.
     *
     * @return {string} The PrimeFaces Extensions version.
     */
	getPrimeFacesExtensionsVersion : function() {
		if (!PrimeFacesExt.VERSION) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
			PrimeFacesExt.VERSION = RegExp('[?&]v=([^&]*)').exec(scriptURI)[1];
		}

		return PrimeFacesExt.VERSION;
	},

    /**
     * Builds a resource URL for a PrimeFaces Extensions resource.
     * 
     * @param {string} name The name of the resource. For example: /core/core.js
     * 
     * @return {string} The resource URL.
     */
	getPrimeFacesExtensionsResource : function(name) {
		var resourceLibrary = PrimeFacesExt.RESOURCE_LIBRARY;
		if (PrimeFacesExt.useUncompressedResources) {
			resourceLibrary = PrimeFacesExt.RESOURCE_LIBRARY_UNCOMPRESSED;
		}

		return PrimeFacesExt.getFacesResource(
				name,
				resourceLibrary,
				PrimeFacesExt.getPrimeFacesExtensionsVersion());
	},

    /**
     * Builds a resource URL for a PrimeFaces Extensions Compressed resource.
     * 
     * @param {string} name The name of the resource. For example: /core/core.js
     * 
     * @return {string} The resource URL.
     */
	getPrimeFacesExtensionsCompressedResource : function(name) {
		return PrimeFacesExt.getFacesResource(
				name,
				PrimeFacesExt.RESOURCE_LIBRARY,
				PrimeFacesExt.getPrimeFacesExtensionsVersion());
	},

    /**
     * Checks if the FacesServlet is mapped with extension mapping. For example: .jsf/.xhtml.
     *
     * @return {boolean} If mapped with extension mapping.
     */
	isExtensionMapping : function() {
		if (!PrimeFacesExt.IS_EXTENSION_MAPPING) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();
			var primeFacesExtensionsScript = 'primefaces-extensions.js';

			PrimeFacesExt.IS_EXTENSION_MAPPING = scriptURI.charAt(scriptURI.indexOf(primeFacesExtensionsScript) + primeFacesExtensionsScript.length) === '.';
		}

		return PrimeFacesExt.IS_EXTENSION_MAPPING;
	},

    /**
     * Checks if the current included scripts are uncompressed.
     *
     * @return {boolean} If uncompresed resources are used.
     */
	useUncompressedResources : function() {
		if (!PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES) {
			var scriptURI = PrimeFacesExt.getPrimeFacesExtensionsScriptURI();

			PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES = scriptURI.indexOf(RESOURCE_LIBRARY_UNCOMPRESSED) !== -1;
		}

		return PrimeFacesExt.USE_UNCOMPRESSED_RESOURCES;
	},

    /**
     * Gets the resource URI of the current included primefaces-extensions.js.
     *
     * @return {string} The resource URI.
     * @protected
     */
	getPrimeFacesExtensionsScriptURI : function() {
		if (!PrimeFacesExt.SCRIPT_URI) {
			PrimeFacesExt.SCRIPT_URI = $('script[src*="' + PrimeFacesExt.RESOURCE_IDENTIFIER + '/primefaces-extensions.js"]').attr('src');
		}

		return PrimeFacesExt.SCRIPT_URI;
	},

	/**
	 * Load a JavaScript file from the server using a GET HTTP request, then execute it.
	 * 
	 * @param {string} url A string containing the URL to which the request is sent.
	 * @param {function} callback A callback function that is executed if the request succeeds.
	 * @param {string} cache Appends a unique timestamp if false.
	 */
	getScript : function(url, callback, cache) {
		$.ajax({
				type: "GET",
				url: url,
				success: callback,
				dataType: "script",
				cache: cache
		});
	},

	/**
	 * The JSF resource identifier.
	 *
	 * @type {string}
	 * @constant
	 */
	RESOURCE_IDENTIFIER : '/javax.faces.resource',

	/**
	 * The name of the PrimeFaces Extensions resource library.
	 *
	 * @type {string}
	 * @constant
	 */
	RESOURCE_LIBRARY : 'primefaces-extensions',

	/**
	 * The name of the PrimeFaces Extensions Uncompressed resource library.
	 *
	 * @type {string}
	 * @constant
	 */
	RESOURCE_LIBRARY_UNCOMPRESSED : 'primefaces-extensions-uncompressed'
};

PrimeFacesExt.behavior = {};
PrimeFacesExt.widget = {};

PrimeFacesExt.behavior.Javascript = function(cfg, ext) {
	return cfg.execute.call(this, cfg.source, cfg.event, cfg.params, ext);
}
;PrimeFacesExt.widget.AjaxStatus = function(id) {
    this.id = id;
    this.jqId = PrimeFaces.escapeClientId(this.id);
    
    this.postConstruct();
}

PrimeFaces.extend(PrimeFacesExt.widget.AjaxStatus, PrimeFaces.widget.BaseWidget);

PrimeFacesExt.widget.AjaxStatus.prototype.bindFacet = function(eventName, facetToShow) {
    var _self = this;

    // jQuery
    jQuery(document).bind(eventName, function() {
        _self.showFacet(facetToShow);
    });

    // JSF standard
    if (eventName == "ajaxError") {
        jsf.ajax.addOnError(this.processAjaxOnErrorFacet(facetToShow));
    } else {
        jsf.ajax.addOnEvent(this.processAjaxOnEventFacet(eventName, facetToShow));
    }    
}

PrimeFacesExt.widget.AjaxStatus.prototype.bindCallback = function(eventName, fn) {
    // jQuery
    jQuery(document).bind(eventName, fn);

    // JSF standard
    if (eventName == "ajaxError") {
        jsf.ajax.addOnError(this.processAjaxOnErrorCallback(fn));
    } else {
        jsf.ajax.addOnEvent(this.processAjaxOnEventCallback(eventName, fn));
    }
}

PrimeFacesExt.widget.AjaxStatus.prototype.processAjaxOnEventFacet = function(eventName, facetToShow) {
    var _self = this;

    function processEvent(data) {
        if (eventName == "ajaxStart" && data.status == "begin") {
            _self.showFacet(facetToShow);
        } else if (eventName == "ajaxComplete" && data.status == "complete") {
            _self.showFacet(facetToShow);
        } else if (eventName == "ajaxSuccess" && data.status == "success") {
            _self.showFacet(facetToShow);
        }
    }

    return processEvent;
}

PrimeFacesExt.widget.AjaxStatus.prototype.processAjaxOnErrorFacet = function(facetToShow) {
    var _self = this;

    function processEvent() {
        _self.showFacet(facetToShow);
    }

    return processEvent;
}

PrimeFacesExt.widget.AjaxStatus.prototype.processAjaxOnEventCallback = function(eventName, fn) {
    function processEvent(data) {
        if (eventName == "ajaxStart" && data.status == "begin") {
            fn();
        } else if (eventName == "ajaxComplete" && data.status == "complete") {
            fn();
        } else if (eventName == "ajaxSuccess" && data.status == "success") {
            fn();
        }
    }

    return processEvent;
}

PrimeFacesExt.widget.AjaxStatus.prototype.processAjaxOnErrorCallback = function(fn) {
    function processEvent() {
        fn();
    }

    return processEvent;
}

PrimeFacesExt.widget.AjaxStatus.prototype.showFacet = function(facetToShow) {
    jQuery(this.jqId).children().hide();
    jQuery(this.jqId + '_' + facetToShow).show();
};/**
 * Resolves the resources for the CKEditor.
 * 
 * @param {string} resource The requested resource from CKEditor.
 * 
 * @return {string} The faces resource URL.
 */
CKEDITOR_GETURL = function(resource) {
	var facesResource;

	//do not resolve
	if (resource.indexOf('?resolve=false') !== -1) {
		facesResource = resource.replace('?resolve=false', '');
	} else {
		//already wrapped?
		var libraryVersionIndex = resource.indexOf('v=' + PrimeFacesExt.getPrimeFacesExtensionsVersion());
		if (libraryVersionIndex !== -1) {
			//look for appended resource
			var appendedResource = resource.substring(libraryVersionIndex + ('v=' + PrimeFacesExt.getPrimeFacesExtensionsVersion()).length);
	
			if (appendedResource.length > 0) {
				//remove append resource from url
				facesResource = resource.substring(0, resource.length - appendedResource.length);
	
				var resourceIdentiferPosition = facesResource.indexOf(PrimeFacesExt.RESOURCE_IDENTIFIER);
				
				if (PrimeFacesExt.isExtensionMapping()) {
					var extensionMappingPosition = facesResource.indexOf(PrimeFacesExt.getRequestUrlExtension());
	
					//extract resource
					var extractedResource = facesResource.substring(resourceIdentiferPosition + PrimeFacesExt.RESOURCE_IDENTIFIER.length, extensionMappingPosition);
	
					facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource(extractedResource + appendedResource);
				} else {
					var questionMarkPosition = facesResource.indexOf('?');
	
					//extract resource
					var extractedResource = facesResource.substring(resourceIdentiferPosition + PrimeFacesExt.RESOURCE_IDENTIFIER.length, questionMarkPosition);
	
					facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource(extractedResource + appendedResource);
				}
			} else {
				facesResource = resource;
			}
		} else {
			facesResource = PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/' + resource);
		}
	}

	return facesResource;
}

/**
 * PrimeFaces Extensions CKEditor Widget
 * 
 * @constructor
 */
PrimeFacesExt.widget.CKEditor = function(id, cfg) {
	this.id = id;
	this.cfg = cfg;
	this.jqId = PrimeFaces.escapeClientId(this.id);
	this.jq = $(this.jqId);

	this.dirtyEventDefined = this.cfg.behaviors && this.cfg.behaviors['dirty'];
	this.changeEventDefined = this.cfg.behaviors && this.cfg.behaviors['change'];
    this.dirtyState = false;

	this.options = {};
	//add widget to ckeditor config, this is required for the save event
	this.options.widget = this;

	if (this.cfg.skin) {
		this.options.skin = this.cfg.skin;
	}
	if (this.cfg.width) {
		this.options.width = this.cfg.width;
	}
	if (this.cfg.height) {
		this.options.height = this.cfg.height;
	}
	if (this.cfg.theme) {
		this.options.theme = this.cfg.theme;
	}
	if (this.cfg.toolbar) {
		this.options.toolbar = this.cfg.toolbar;
	}
	if (this.cfg.readOnly) {
		this.options.readOnly = this.cfg.readOnly;
	}
	if (this.cfg.interfaceColor) {
		this.options.uiColor = this.cfg.interfaceColor;
	}
	if (this.cfg.language) {
		this.options.language = this.cfg.language;
	}
	if (this.cfg.defaultLanguage) {
		this.options.defaultLanguage = this.cfg.defaultLanguage;
	}
	if (this.cfg.contentsCss) {
		this.options.contentsCss = this.cfg.contentsCss;
	}
	if (this.cfg.customConfig) {
		this.options.customConfig = this.cfg.customConfig + "?resolve=false";
	}

	//check if ckeditor is already included
	if (typeof(CKEDITOR) == 'undefined') {
		var ckEditorScriptURI =
			PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/ckeditor.js');
		
		var jQueryAdapterScriptURI =
			PrimeFacesExt.getPrimeFacesExtensionsCompressedResource('/ckeditor/adapters/jquery.js');		

		//load ckeditor
		PrimeFacesExt.getScript(ckEditorScriptURI, $.proxy(function(data, textStatus) {

			//load jquery adapter
			PrimeFacesExt.getScript(jQueryAdapterScriptURI, $.proxy(function(data, textStatus) {

				//overwrite save button and init editor
				this.overwriteSaveButton();
				this.initialize();

			}, this), true);

		}, this), true);

	} else {
		this.initialize();
	}
}

PrimeFaces.extend(PrimeFacesExt.widget.CKEditor, PrimeFaces.widget.BaseWidget);

/**
 * Initializes the CKEditor instance.
 * This method will be called when the resources for the CKEditor are loaded.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.initialize = function() {
	//remove old instances if required
	var oldInstance = CKEDITOR.instances[this.id];
	if (oldInstance) {
		oldInstance.destroy(true);
	}

	//initialize ckeditor after all resources were loaded
	this.jq.ckeditor($.proxy(function() { this.initialized(); }, this), this.options);
}

/**
 * Overwrites the save button.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.overwriteSaveButton = function() {
	//overwrite save button
	CKEDITOR.plugins.registered['save'] = {
		init : function(editor) {
			//get widget
			var widget = editor.config.widget;
			var command = editor.addCommand('save', {
				modes : { wysiwyg:1, source:1 },
				exec : function(editor) {
					if (widget.cfg.behaviors) {
						var saveCallback = widget.cfg.behaviors['save'];
					    if (saveCallback) {
					    	var ext = {
					    			params: {}
					    	};

					    	saveCallback.call(widget, null, ext);
					    }
					}
				}
			});
			editor.ui.addButton('Save', {label : editor.lang.save, command : 'save' });
		}
	}
}

/**
 * This method will be called when the CKEditor was initialized. 
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.initialized = function() {
	//get instance
	this.instance = this.jq.ckeditorGet();
	
	//fire initialize event
	this.fireEvent('initialize');
	
	//register blur and focus event
    this.instance.on('blur', $.proxy(function() { this.handleBlur(); }, this));
    this.instance.on('focus', $.proxy(function() { this.fireEvent('focus'); }, this));

    //register callbacks for change event
    if (this.changeEventDefined || this.dirtyEventDefined) {  
    	
    	this.bindChangeEventsForWYSIWYGMode();
    	this.bindCommonChangeEvents();

    	//changes to WYSIWYG mode
    	this.instance.on('contentDom', $.proxy(function() {
    		this.bindChangeEventsForWYSIWYGMode();
    		this.fireEvent('wysiwygMode');
      	}, this));
 
    	//changes to source mode
    	this.instance.on('mode', $.proxy(function(event) {
			if (this.instance.mode != 'source') {
				return;    	
			}
			this.bindChangeEventsForSourceMode();
			this.fireEvent('sourceMode');
    	}, this));
    }

    if (this.dirtyEventDefined && this.cfg.checkDirtyInterval !== 0) {
	    this.dirtyCheckInterval = setInterval($.proxy(this.checkDirtyFromTimer, this), this.cfg.checkDirtyInterval);
    }

    this.postConstruct();
}

/**
 * Binds the common events, which will be used for dirty/change checking.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.bindCommonChangeEvents = function() {
	this.instance.on('paste', $.proxy(this.checkDirty, this));
	this.instance.getCommand('undo').on('afterUndo', $.proxy(this.checkDirty, this));
	this.instance.getCommand('redo').on('afterRedo', $.proxy(this.checkDirty, this));

	this.instance.on('saveSnapshot', $.proxy(function(event) {
		if (!event.data || !event.data.contentOnly) {
			this.checkDirty();
		}
	}, this));

	this.instance.on('afterCommandExec', $.proxy(function(event) {
		if (event.data.name == 'source') {
			return;
		}

		if (event.data.command.canUndo !== false) {
			this.checkDirty();
		}
	}, this));
};

/**
 * Binds the events for the WYSIWYG mode, which will be used for dirty/change checking.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.bindChangeEventsForWYSIWYGMode = function() {
    this.instance.document.on('drop', $.proxy(this.checkDirty, this));
    this.instance.document.getBody().on('drop', $.proxy(this.checkDirty, this));
    this.instance.document.on('keydown', $.proxy(function(event) {
    	//do not capture ctrl and meta keys
    	if (event.data.$.ctrlKey ||event.data.$.metaKey) {
    		return;
    	}

    	//filter movement keys and related
    	var keyCode = event.data.$.keyCode;
    	if (keyCode == 8 || keyCode == 13 || keyCode == 32
    			|| ( keyCode >= 46 && keyCode <= 90)
    			|| ( keyCode >= 96 && keyCode <= 111)
    			|| ( keyCode >= 186 && keyCode <= 222)) {
    		this.checkDirty();
    	}
    }, this));
};

/**
 * Binds the events for the source mode, which will be used for dirty/change checking.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.bindChangeEventsForSourceMode = function() {
	this.instance.textarea.on('drop', $.proxy(this.checkDirty, this));
	this.instance.textarea.on('input', $.proxy(this.checkDirty, this));
	/*
	this.instance.textarea.on('keydown', $.proxy(function(event) {
		//do not capture ctrl and meta keys
		if (!event.data.$.ctrlKey && !event.data.$.metaKey) {
			this.checkDirty();
		}
	}, this));;
	*/
};

/**
 * Sets the dirty state.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.checkDirty = function() {
    if (this.isDirty()) {
        this.instance.resetDirty();

        if (this.dirtyState === false) {
        	this.dirtyState = true;
        }
        
        if (this.dirtyEventDefined) {
        	this.fireEvent('dirty');
        }
    }
};

/**
 * Restores the dirtyState and calls check checkDirty();
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.checkDirtyFromTimer = function() {
	this.dirtyState = false;
	this.checkDirty();
};

/**
 * Handles the blur event and fires the change event if required.
 *
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.handleBlur = function() {
	this.fireEvent('blur');

	if (this.changeEventDefined) {
		if (this.dirtyState) {
		    this.instance.resetDirty();
		    this.fireEvent('change');
		}

		this.dirtyState = false;
	}
};

/**
 * This method fires an event if the behavior was defined.
 *
 * @param {string} eventName The name of the event
 * @protected
 */
PrimeFacesExt.widget.CKEditor.prototype.fireEvent = function(eventName) {
	if (this.cfg.behaviors) {
		var callback = this.cfg.behaviors[eventName];
	    if (callback) {
	    	var ext = {
	    			params: {}
	    	};

	    	callback.call(this, null, ext);
	    }
	}
}

/**
 * Destroys the CKEditor instance.
 */
PrimeFacesExt.widget.CKEditor.prototype.destroy = function() {
	if (this.dirtyCheckInterval) {
		clearInterval(this.dirtyCheckInterval);
	}

    if (this.instance) {
        this.instance.destroy(true);
        this.instance = null;
    }

    this.jq.show();
}

/**
 * Checks if the editor is in dirty state.
 */
PrimeFacesExt.widget.CKEditor.prototype.isDirty = function() {
	return this.dirtyState || this.instance.checkDirty();
}

/**
 * Sets readOnly to the CKEditor.
 */
PrimeFacesExt.widget.CKEditor.prototype.setReadOnly = function(readOnly) {
    this.instance.setReadOnly(readOnly !== false);
}

/**
 * Checks if the CKEditor is readOnly.
 */
PrimeFacesExt.widget.CKEditor.prototype.isReadOnly = function() {
    return this.instance.readOnly;
}

/**
 * Indicates that the editor instance has focus.
 */
PrimeFacesExt.widget.CKEditor.prototype.hasFocus = function() {
    return this.instance.focusManager.hasFocus;
}

/**
 * Returns the CKEditor instance.
 */
PrimeFacesExt.widget.CKEditor.prototype.getEditorInstance = function() {
    return this.instance;
}
;/**
 * PrimeFaces Extensions ImageRotateAndResize Widget.
 *
 * @constructor
 */
PrimeFacesExt.widget.ImageRotateAndResize = function(id, cfg) {
	this.id = id;
	this.cfg = cfg;
	this.initialized = false;
}

/**
 * Initializes the settings.
 *
 * @protected
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.initializeLazy = function() {
	if (!this.initialized) {
		this.target = $(this.cfg.target)[0];
		this.imageSrc = this.target.src;
		this.imageWidth = this.target.width;
		this.imageHeight = this.target.height;
		this.degree = 0;
		this.newImageWidth = this.target.width;
		this.newImageHeight = this.target.height;
		this.initialized = true;
	}
}

/**
 * Reloads the widget.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.reload = function() {
	this.initialized = false;
	this.initializeLazy();
}

/**
 * Rotates the image to the left direction.
 * 
 * @param {number} degree The degree.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.rotateLeft = function(degree) {
	this.initializeLazy();

	if ((this.degree - degree) <= 0) {
		this.degree = 360 - ((this.degree - degree) * -1);
	}
	else {
		this.degree -= degree;
	}

	this.redrawImage();
	this.fireRotateEvent();
}

/**
 * Rotates the image to the right direction.
 * 
 * @param {number} degree The degree.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.rotateRight = function(degree) {
	this.initializeLazy();

	if ((this.degree + degree) >= 360) {
		this.degree = (this.degree + degree) - 360;
	}
	else {
		this.degree += degree;
	}

	this.redrawImage();
	this.fireRotateEvent();
}

/**
 * Resizes the image to the given width and height.
 * 
 * @param {number} width The new width of the image.
 * @param {number} height The new height of the image.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.resize = function(width, height) {
	this.initializeLazy();

	this.newImageWidth = width;
	this.newImageHeight = height;

	this.redrawImage();
	this.fireResizeEvent();
}

/**
 * Scales the image with the given factor.
 * 
 * @param {number} scaleFactor The scale factor. For example: 1.1 = scales the image to 110% size.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.scale = function(scaleFactor) {
	this.initializeLazy();

	this.newImageWidth = this.newImageWidth * scaleFactor;
	this.newImageHeight = this.newImageHeight * scaleFactor;

	this.redrawImage();
	this.fireResizeEvent();
}

/**
 * Restores the default image.
 * It also fires the rotate and resize event with the default values.
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.restoreDefaults = function() {
	this.initializeLazy();

	this.newImageWidth = this.imageWidth;
	this.newImageHeight = this.imageHeight;
	this.degree = 0;

	this.redrawImage();
	this.fireResizeEvent();
	this.fireRotateEvent();
}

/**
 * Redraws the image.
 *
 * @protected
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.redrawImage = function() {
	var rotation;
	if (this.degree >= 0) {
		rotation = Math.PI * this.degree / 180;
	} else {
		rotation = Math.PI * (360 + this.degree) / 180;
	}

	var cos = Math.cos(rotation);
	var sin = Math.sin(rotation);

	//check for < IE9, otherwise use canvas
	if ($.browser.msie && parseInt($.browser.version) <= 8) {
		//create new image
		var image = document.createElement('img');
		image.src = this.imageSrc;
		
		//set new size
		image.height = this.newImageHeight;
		image.width = this.newImageWidth;

		//apply rotation for IE
		image.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=" + cos + ",M12=" + (sin * -1) + ",M21=" + sin + ",M22=" + cos + ",SizingMethod='auto expand')";	
		
		//replace old image with new generated one
		image.id = this.target.id;
		this.target.parentNode.replaceChild(image, this.target);
		this.target = image;
	} else {
		//create canvas instead of img
		var canvas = document.createElement('canvas');

		//new image with new size
		var newImage = new Image();
		newImage.src = this.imageSrc;
		newImage.width = this.newImageWidth;
		newImage.height = this.newImageHeight;

		//rotate
		canvas.style.width = canvas.width = Math.abs(cos * newImage.width) + Math.abs(sin * newImage.height);
		canvas.style.height = canvas.height = Math.abs(cos * newImage.height) + Math.abs(sin * newImage.width);

		var context = canvas.getContext('2d');
		context.save();
		
		if (rotation <= Math.PI/2) {
			context.translate(sin * newImage.height, 0);
		} else if (rotation <= Math.PI) {
			context.translate(canvas.width, (cos * -1) * newImage.height);
		} else if (rotation <= 1.5 * Math.PI) {
			context.translate((cos * -1) * newImage.width, canvas.height);
		} else {
			context.translate(0, (sin * -1) * newImage.width);
		}

		context.rotate(rotation);
		context.drawImage(newImage, 0, 0, newImage.width, newImage.height);
		context.restore();	

		//replace image with canvas and set src attribute
		canvas.id = this.target.id;
		canvas.src = this.target.src;
		this.target.parentNode.replaceChild(canvas, this.target);
		this.target = canvas;
	}
}

/**
 * Fires the rotate event.
 *
 * @protected
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.fireRotateEvent = function() {
	if (this.cfg.behaviors) {
		var callback = this.cfg.behaviors['rotate'];
	    if (callback) {
	    	var ext = {
	    			params: {}
	    	};
	
	    	ext.params[this.id + '_degree'] = this.degree;
	   
	    	callback.call(this, null, ext);
	    }
	}
}

/**
 * Fires the resize event.
 *
 * @protected
 */
PrimeFacesExt.widget.ImageRotateAndResize.prototype.fireResizeEvent = function() {
	if (this.cfg.behaviors) {
		var callback = this.cfg.behaviors['resize'];
	    if (callback) {
	    	var ext = {
	    			params: {}
	    	};
	    	
	    	ext.params[this.id + '_width'] = this.newImageWidth;
	    	ext.params[this.id + '_height'] = this.newImageHeight;
	    	
	    	callback.call(this, null, ext);
	    }
	}
}
